@@ -23,6 +23,12 @@ typedef struct icmp_ram		/* RFC 1256 */
 	u16_t	iram_lt;
 } icmp_ram_t;
 
+typedef struct icmp_pp
+{
+	u8_t	ipp_ptr;
+	u8_t	ipp_unused[3];
+} icmp_pp_t;
+
 typedef struct icmp_hdr
 {
 	u8_t ih_type, ih_code;
@@ -33,6 +39,7 @@ typedef struct icmp_hdr
 		icmp_id_seq_t ihh_idseq;
 		ipaddr_t ihh_gateway;
 		icmp_ram_t ihh_ram;
+		icmp_pp_t ihh_pp;
 	} ih_hun;
 	union
 	{
@@ -42,3 +49,7 @@ typedef struct icmp_hdr
 } icmp_hdr_t;
 
 #endif /* __SERVER__IP__GEN__ICMP_HDR_H__ */
+
+/*
+ * $PchId: icmp_hdr.h,v 1.4 1995/11/17 22:28:58 philip Exp $
+ */
