@@ -12,7 +12,6 @@
 #include <minix/config.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <ar.h>
 #include <fcntl.h>
 #include <limits.h>
 #include <grp.h>
@@ -65,8 +64,9 @@ _PROTOTYPE(void Draw_Offset , (de_state *s ));
 _PROTOTYPE(void Word_Pointers , (off_t old_addr , off_t new_addr ));
 _PROTOTYPE(void Block_Pointers , (off_t old_addr , off_t new_addr ));
 _PROTOTYPE(void Map_Pointers , (off_t old_addr , off_t new_addr ));
-_PROTOTYPE(void Print_Number , (word_t number , int output_base ));
-_PROTOTYPE(void Draw_Zone_Numbers , (de_state *s , struct inode *inode , int zindex , int zrow ));
+_PROTOTYPE(void Print_Number , (Word_t number , int output_base ));
+_PROTOTYPE(void Draw_Zone_Numbers , (de_state *s , struct inode *inode ,
+						int zindex , int zrow ));
 
 
 
@@ -523,14 +523,16 @@ void Draw_Words( s )
 
 
 char *super_block_info[] =  {	"number of inodes",
-				"number of zones",
+				"V1 number of zones",
 				"inode bit map blocks",
 				"zone bit map blocks",
 				"first data zone",
-				"blocks per zone shift",
+				"blocks per zone shift & flags",
 				"maximum file size",
 				"",
-				"magic number"  };
+				"magic number",
+				"fsck magic number",
+				"V2 number of zones"  };
 
 
 void Draw_Info( s )
@@ -543,7 +545,7 @@ void Draw_Info( s )
 
 
   if ( s->is_fs  &&  s->block == SUPER_BLOCK  &&  page == 0 )
-      for ( i = 0;  i < 9;  ++i )
+      for ( i = 0;  i < 11;  ++i )
  	{
 	Goto( INFO_COLUMN, INFO_LINE + i );
 	printf( "%s", super_block_info[ i ] );
@@ -679,17 +681,9 @@ void Draw_Info( s )
 	unsigned magic  = ((s->buffer[1] & 0xff) << 8) | (s->buffer[0] & 0xff);
 	unsigned second = ((s->buffer[3] & 0xff) << 8) | (s->buffer[2] & 0xff);
 
-        /*  Is this block the start of an "ar" archive?  */
-
-	if ( magic == (unsigned) ARMAG )
-	  {
-          Goto( INFO_COLUMN, INFO_LINE );
-	  printf( "\"ar\" archive" );
-	  }
-
 	/*  Is this block the start of an executable file?  */
 
-	else if ( magic == (unsigned) A_OUT )
+	if ( magic == (unsigned) A_OUT )
 	  {
           Goto( INFO_COLUMN, INFO_LINE );
 	  printf( "executable" );
@@ -1158,10 +1152,10 @@ void Draw_Zone_Numbers( s, inode, zindex, zrow )
 	++zindex, zrow += s->zone_num_size / sizeof (word_t) )
     {
     Goto( INFO_COLUMN, INFO_LINE + zrow );
-    if ( zindex < inode->i_ndzones )
+    if ( zindex < s->ndzones )
       printf( "zone %d", zindex );
     else
-      printf( "%sindirect", plurals[ zindex - inode->i_ndzones ] );
+      printf( "%sindirect", plurals[ zindex - s->ndzones ] );
     if ( s->magic != SUPER_MAGIC )
       {
       zone = inode->i_zone[ zindex ];
