@@ -1,13 +1,15 @@
 /*
 icmp.c
+
+Copyright 1995 Philip Homburg
 */
 
 #include "inet.h"
 #include "buf.h"
+#include "event.h"
 #include "type.h"
 
 #include "assert.h"
-#include "clock.h"
 #include "icmp.h"
 #include "icmp_lib.h"
 #include "io.h"
@@ -15,7 +17,7 @@ icmp.c
 #include "ip_int.h"
 #include "ipr.h"
 
-INIT_PANIC();
+THIS_FILE
 
 typedef struct icmp_port
 {
@@ -72,7 +74,11 @@ FORWARD acc_t *make_repl_ip ARGS(( ip_hdr_t *ip_hdr,
 FORWARD void enqueue_pack ARGS(( icmp_port_t *icmp_port,
 	acc_t *reply_ip_hdr ));
 FORWARD void icmp_write ARGS(( icmp_port_t *icmp_port ));
-FORWARD void icmp_buffree ARGS(( int priority, size_t reqsize ));
+FORWARD void icmp_buffree ARGS(( int priority ));
+FORWARD acc_t *icmp_err_pack ARGS(( acc_t *pack, icmp_hdr_t **icmp_hdr ));
+#ifdef BUF_CONSISTENCY_CHECK
+FORWARD void icmp_bufcheck ARGS(( void ));
+#endif
 
 PUBLIC void icmp_init()
 {
@@ -84,12 +90,18 @@ PUBLIC void icmp_init()
 	for (i= 0, icmp_port= icmp_port_table; i<ICMP_PORT_NR; i++,
 		icmp_port++)
 	{
+#if ZERO
 		icmp_port->icp_flags= ICPF_EMPTY;
 		icmp_port->icp_state= ICPS_BEGIN;
+#endif
 		icmp_port->icp_ipport= i;
 	}
 
+#ifndef BUF_CONSISTENCY_CHECK
 	bf_logon(icmp_buffree);
+#else
+	bf_logon(icmp_buffree, icmp_bufcheck);
+#endif
 
 	for (i= 0, icmp_port= icmp_port_table; i<ICMP_PORT_NR; i++,
 		icmp_port++)
@@ -108,12 +120,11 @@ icmp_port_t *icmp_port;
 		icmp_port->icp_head_queue= 0;
 		icmp_port->icp_ipfd= ip_open (icmp_port->icp_ipport,
 			icmp_port-icmp_port_table, icmp_getdata,
-			icmp_putdata);
+			icmp_putdata, 0);
 		if (icmp_port->icp_ipfd<0)
 		{
-			where();
-			printf("unable to open ip_port %d\n",
-				icmp_port->icp_ipport);
+			DBLOCK(1, printf("unable to open ip_port %d\n",
+				icmp_port->icp_ipport));
 			break;
 		}
 		icmp_port->icp_state= ICPS_IPOPT;
@@ -124,13 +135,8 @@ icmp_port_t *icmp_port;
 			icmp_port->icp_flags |= ICPF_SUSPEND;
 			break;
 		}
-		else if (result<0)
-		{
-			where();
-			printf("ip_ioctl (.., NWIOSIPOPT)= %d\n",
-				result);
-			break;
-		}
+		assert(result == NW_OK);
+
 		/* falls through */
 	case ICPS_IPOPT:
 		icmp_port->icp_state= ICPS_MAIN;
@@ -138,8 +144,8 @@ icmp_port_t *icmp_port;
 		icmp_read(icmp_port);
 		break;
 	default:
-		where();
-		printf("unknown state %d\n", icmp_port->icp_state);
+		DBLOCK(1, printf("unknown state %d\n",
+			icmp_port->icp_state));
 		break;
 	}
 }
@@ -166,12 +172,9 @@ int for_ioctl;
 			result= (int)offset;
 			if (result<0)
 			{
-				where();
-				printf("got write error %d\n", result);
+				DBLOCK(1, printf("got write error %d\n",
+					result));
 			}
-#if DEBUG & 256
- { where(); printf("ip_write completed\n"); }
-#endif
 			if (icmp_port->icp_flags & ICPF_WRITE_SP)
 			{
 				icmp_port->icp_flags &=
@@ -188,6 +191,7 @@ int for_ioctl;
 		if (!count)
 		{
 			result= (int)offset;
+			assert(result == NW_OK);
 			if (result < 0)
 			{
 				icmp_port->icp_state= ICPS_ERROR;
@@ -203,13 +207,14 @@ assert (count == sizeof (*ipopt));
 assert (data->acc_length == sizeof(*ipopt));
 		ipopt= (nwio_ipopt_t *)ptr2acc_data(data);
 		ipopt->nwio_flags= NWIO_COPY | NWIO_EN_LOC |
-			NWIO_EN_BROAD | NWIO_REMANY | NWIO_PROTOSPEC |
+			NWIO_EN_BROAD |
+			NWIO_REMANY | NWIO_PROTOSPEC |
 			NWIO_HDR_O_ANY | NWIO_RWDATALL;
 		ipopt->nwio_proto= IPPROTO_ICMP;
 		return data;
 	default:
-		where();
-		printf("unknown state %d\n", icmp_port->icp_state);
+		DBLOCK(1, printf("unknown state %d\n",
+			icmp_port->icp_state));
 		return 0;
 	}
 }
@@ -233,8 +238,8 @@ assert (!for_ioctl);
 			result= (int)offset;
 			if (result<0)
 			{
-				where();
-				printf("got read error %d\n", result);
+				DBLOCK(1, printf("got read error %d\n",
+					result));
 			}
 			if (icmp_port->icp_flags & ICPF_READ_SP)
 			{
@@ -250,8 +255,8 @@ assert (!for_ioctl);
 	switch (icmp_port->icp_state)
 	{
 	default:
-		where();
-		printf("unknown state %d\n", icmp_port->icp_state);
+		DBLOCK(1, printf("unknown state %d\n",
+			icmp_port->icp_state));
 		return 0;
 	}
 }
@@ -279,46 +284,71 @@ assert (!(icmp_port->icp_flags & (ICPF_READ_IP|ICPF_READ_SP) ||
 	}
 }
 
-PUBLIC void icmp_frag_ass_tim(pack)
+PUBLIC void icmp_snd_time_exceeded(port_nr, pack, code)
+int port_nr;
 acc_t *pack;
+int code;
 {
-	ip_warning(( "icmp_frag_ass() called" ));
-	bf_afree(pack);
+	acc_t *icmp_acc;
+	icmp_hdr_t *icmp_hdr;
+	icmp_port_t *icmp_port;
+
+	assert(0 <= port_nr && port_nr < ICMP_PORT_NR);
+	icmp_port= &icmp_port_table[port_nr];
+	pack= icmp_err_pack(pack, &icmp_hdr);
+	if (pack == NULL)
+		return;
+	icmp_hdr->ih_type= ICMP_TYPE_TIME_EXCEEDED;
+	icmp_hdr->ih_code= code;
+	icmp_hdr->ih_chksum= ~oneC_sum(~icmp_hdr->ih_chksum,
+		(u16_t *)&icmp_hdr->ih_type, 2);
+	enqueue_pack(icmp_port, pack);
 }
 
-PUBLIC void icmp_getnetmask(ip_port)
-int ip_port;
+PUBLIC void icmp_snd_redirect(port_nr, pack, code, gw)
+int port_nr;
+acc_t *pack;
+int code;
+ipaddr_t gw;
 {
-	ip_port_t *port;
+	acc_t *icmp_acc;
+	icmp_hdr_t *icmp_hdr;
+	icmp_port_t *icmp_port;
 
-#if DEBUG & 256
- { where(); printf("icmp.c: icmp_getnetmask(ip_port= %d)\n", ip_port); }
-#endif
-	port= &ip_port_table[ip_port];
-#if DEBUG & 2
- { where(); printf ("icmp_getnetmask() NOT implemented\n"); }
-#endif
-
-	port->ip_netmask= HTONL(0xffffff00L);
-	port->ip_flags |= IPF_NETMASKSET;
-#if DEBUG & 256
- { where(); printf("icmp.c: setting netmask to "); 
-   writeIpAddr(port->ip_netmask); printf("\n"); }
-#endif
+	assert(0 <= port_nr && port_nr < ICMP_PORT_NR);
+	icmp_port= &icmp_port_table[port_nr];
+	pack= icmp_err_pack(pack, &icmp_hdr);
+	if (pack == NULL)
+		return;
+	icmp_hdr->ih_type= ICMP_TYPE_REDIRECT;
+	icmp_hdr->ih_code= code;
+	icmp_hdr->ih_hun.ihh_gateway= gw;
+	icmp_hdr->ih_chksum= ~oneC_sum(~icmp_hdr->ih_chksum,
+		(u16_t *)&icmp_hdr->ih_type, 2);
+	icmp_hdr->ih_chksum= ~oneC_sum(~icmp_hdr->ih_chksum,
+		(u16_t *)&icmp_hdr->ih_hun.ihh_gateway, 4);
+	enqueue_pack(icmp_port, pack);
 }
 
-PUBLIC void icmp_dont_frag(pack)
+PUBLIC void icmp_snd_unreachable(port_nr, pack, code)
+int port_nr;
 acc_t *pack;
+int code;
 {
-printf ("icmp_dont_frag() called\n");
-	bf_afree(pack);
-}
+	acc_t *icmp_acc;
+	icmp_hdr_t *icmp_hdr;
+	icmp_port_t *icmp_port;
 
-PUBLIC void icmp_ttl_exceded(pack)
-acc_t *pack;
-{
-printf ("icmp_ttl_execeded() called\n");
-	bf_afree(pack);
+	assert(0 <= port_nr && port_nr < ICMP_PORT_NR);
+	icmp_port= &icmp_port_table[port_nr];
+	pack= icmp_err_pack(pack, &icmp_hdr);
+	if (pack == NULL)
+		return;
+	icmp_hdr->ih_type= ICMP_TYPE_DST_UNRCH;
+	icmp_hdr->ih_code= code;
+	icmp_hdr->ih_chksum= ~oneC_sum(~icmp_hdr->ih_chksum,
+		(u16_t *)&icmp_hdr->ih_type, 2);
+	enqueue_pack(icmp_port, pack);
 }
 
 PRIVATE void process_data(icmp_port, data)
@@ -331,11 +361,13 @@ acc_t *data;
 	int ip_hdr_len;
 	size_t pack_len;
 
-#if DEBUG & 256
- { where(); printf("got an icmp packet\n"); }
-#endif
+	/* Align entire packet */
+	data= bf_align(data, BUF_S, 4);
+
 	data= bf_packIffLess(data, IP_MIN_HDR_SIZE);
 	ip_hdr= (ip_hdr_t *)ptr2acc_data(data);
+	DIFBLOCK(0x10, (ip_hdr->ih_dst & HTONL(0xf0000000)) == HTONL(0xe0000000),
+		printf("got multicast packet\n"));
 	ip_hdr_len= (ip_hdr->ih_vers_ihl & IH_IHL_MASK) << 2;
 
 	if (ip_hdr_len>IP_MIN_HDR_SIZE)
@@ -348,9 +380,7 @@ acc_t *data;
 	pack_len -= ip_hdr_len;
 	if (pack_len < ICMP_MIN_HDR_LEN)
 	{
-#if DEBUG
- { where(); printf("got an incomplete icmp packet\n"); }
-#endif
+		DBLOCK(1, printf("got an incomplete icmp packet\n"));
 		bf_afree(data);
 		return;
 	}
@@ -360,28 +390,29 @@ acc_t *data;
 	icmp_data= bf_packIffLess (icmp_data, ICMP_MIN_HDR_LEN);
 	icmp_hdr= (icmp_hdr_t *)ptr2acc_data(icmp_data);
 
-
 	if ((u16_t)~icmp_pack_oneCsum(icmp_data))
 	{
-#if DEBUG
- { where(); printf("got packet with bad checksum (= 0x%x)\n",
-	(u16_t)~icmp_pack_oneCsum(icmp_data)); }
-#endif
+		DBLOCK(1, printf(
+			"got packet with bad checksum (= 0x%x, 0x%x)\n",
+			icmp_hdr->ih_chksum,
+			(u16_t)~icmp_pack_oneCsum(icmp_data)));
 		bf_afree(data);
 		bf_afree(icmp_data);
 		return;
 	}
+
 	switch (icmp_hdr->ih_type)
 	{
 	case ICMP_TYPE_ECHO_REPL:
-#if DEBUG
- { where(); printf("got an icmp echo reply\n"); }
-#endif
 		break;
 	case ICMP_TYPE_DST_UNRCH:
 		icmp_dst_unreach (icmp_port, data, ip_hdr_len, ip_hdr,
 			icmp_data, pack_len, icmp_hdr);
 		break;
+	case ICMP_TYPE_SRC_QUENCH:
+		/* Ignore src quench ICMPs */
+		DBLOCK(2, printf("ignoring SRC QUENCH ICMP.\n"));
+		break;
 	case ICMP_TYPE_REDIRECT:
 		icmp_redirect (icmp_port, ip_hdr, icmp_data, pack_len,
 			icmp_hdr);
@@ -394,15 +425,16 @@ acc_t *data;
 		icmp_router_advertisement(icmp_port, icmp_data, pack_len, 
 			icmp_hdr);
 		break;
+	case ICMP_TYPE_ROUTE_SOL:
+		break;	/* Should be handled by a routing deamon. */
 	case ICMP_TYPE_TIME_EXCEEDED:
 		icmp_time_exceeded (icmp_port, data, ip_hdr_len, ip_hdr,
 			icmp_data, pack_len, icmp_hdr);
 		break;
 	default:
-#if DEBUG
- { where(); printf("got an unknown icmp (%d) from ", icmp_hdr->ih_type); 
-	writeIpAddr(ip_hdr->ih_src); printf("\n"); }
-#endif
+		DBLOCK(1, printf("got an unknown icmp (%d) from ",
+			icmp_hdr->ih_type); 
+			writeIpAddr(ip_hdr->ih_src); printf("\n"));
 		break;
 	}
 	bf_afree(data);
@@ -419,33 +451,25 @@ icmp_hdr_t *icmp_hdr;
 {
 	acc_t *repl_ip_hdr, *repl_icmp;
 	icmp_hdr_t *repl_icmp_hdr;
-	u32_t tmp_chksum;
+	i32_t tmp_chksum;
 	u16_t u16;
 
 	if (icmp_hdr->ih_code != 0)
 	{
-#if DEBUG
- { where(); printf("got an icmp echo request with unknown code (%d)\n",
-		icmp_hdr->ih_code); }
-#endif
+		DBLOCK(1,
+		printf("got an icmp echo request with unknown code (%d)\n",
+			icmp_hdr->ih_code));
 		bf_afree(ip_data);
 		bf_afree(icmp_data);
 		return;
 	}
 	if (icmp_len < ICMP_MIN_HDR_LEN + sizeof(icmp_id_seq_t))
 	{
-#if DEBUG
- { where(); printf("got an incomplete icmp echo request\n"); }
-#endif
+		DBLOCK(1, printf("got an incomplete icmp echo request\n"));
 		bf_afree(ip_data);
 		bf_afree(icmp_data);
 		return;
 	}
-#if DEBUG & 256
- { where(); printf("got an icmp echo request, ident= %u, seq= %u\n",
-	ntohs(icmp_hdr->ih_hun.ihh_idseq.iis_id),
-	ntohs(icmp_hdr->ih_hun.ihh_idseq.iis_seq)); }
-#endif
 	repl_ip_hdr= make_repl_ip(ip_hdr, ip_len);
 	repl_icmp= bf_memreq (ICMP_MIN_HDR_LEN);
 assert (repl_icmp->acc_length == ICMP_MIN_HDR_LEN);
@@ -453,11 +477,17 @@ assert (repl_icmp->acc_length == ICMP_MIN_HDR_LEN);
 	repl_icmp_hdr->ih_type= ICMP_TYPE_ECHO_REPL;
 	repl_icmp_hdr->ih_code= 0;
 
-	tmp_chksum= ~icmp_hdr->ih_chksum - *(u16_t *)&icmp_hdr->ih_type+
+	DBLOCK(2,
+	printf("ih_chksum= 0x%x, ih_type= 0x%x, repl->ih_type= 0x%x\n",
+		icmp_hdr->ih_chksum, *(u16_t *)&icmp_hdr->ih_type, 
+		*(u16_t *)&repl_icmp_hdr->ih_type));
+	tmp_chksum= (~icmp_hdr->ih_chksum & 0xffff) - 
+		(i32_t)*(u16_t *)&icmp_hdr->ih_type+
 		*(u16_t *)&repl_icmp_hdr->ih_type;
 	tmp_chksum= (tmp_chksum >> 16) + (tmp_chksum & 0xffff);
 	tmp_chksum= (tmp_chksum >> 16) + (tmp_chksum & 0xffff);
 	repl_icmp_hdr->ih_chksum= ~tmp_chksum;
+	DBLOCK(2, printf("sending chksum 0x%x\n", repl_icmp_hdr->ih_chksum));
 
 	repl_ip_hdr->acc_next= repl_icmp;
 	repl_icmp->acc_next= bf_cut (icmp_data, ICMP_MIN_HDR_LEN,
@@ -485,7 +515,6 @@ acc_t *icmp_pack;
 	odd_byte= FALSE;
 	for (; icmp_pack; icmp_pack= icmp_pack->acc_next)
 	{
-		
 		data_ptr= ptr2acc_data(icmp_pack);
 		length= icmp_pack->acc_length;
 
@@ -524,9 +553,7 @@ int ip_len;
 
 	if (ip_len>IP_MIN_HDR_SIZE)
 	{
-#if DEBUG
- { where(); printf("ip_hdr options NOT supported (yet?)\n"); }
-#endif
+		DBLOCK(1, printf("ip_hdr options NOT supported (yet?)\n"));
 		ip_len= IP_MIN_HDR_SIZE;
 	}
 
@@ -561,8 +588,9 @@ acc_t *reply_ip_hdr;
 	else
 	{
 		icmp_port->icp_head_queue= reply_ip_hdr;
-		icmp_port->icp_tail_queue= reply_ip_hdr;
 	}
+	reply_ip_hdr->acc_ext_link= NULL;
+	icmp_port->icp_tail_queue= reply_ip_hdr;
 
 	if (!(icmp_port->icp_flags & ICPF_WRITE_IP))
 		icmp_write(icmp_port);
@@ -573,74 +601,72 @@ icmp_port_t *icmp_port;
 {
 	int result;
 
-assert (!(icmp_port->icp_flags & (ICPF_WRITE_IP|ICPF_WRITE_SP) || 
-	(icmp_port->icp_flags & (ICPF_WRITE_IP|ICPF_WRITE_SP)) ==
-	(ICPF_WRITE_IP|ICPF_WRITE_SP)));
+assert (!(icmp_port->icp_flags & ICPF_WRITE_IP));
 
-	for (;icmp_port->icp_head_queue;)
+	while (icmp_port->icp_head_queue != NULL)
 	{
+		assert(icmp_port->icp_write_pack == NULL);
 		icmp_port->icp_write_pack= icmp_port->icp_head_queue;
 		icmp_port->icp_head_queue= icmp_port->icp_head_queue->
 			acc_ext_link;
 
 		icmp_port->icp_flags |= ICPF_WRITE_IP;
-		icmp_port->icp_flags &= ~ICPF_WRITE_SP;
 
-#if DEBUG & 256
- { where(); printf("calling ip_write\n"); }
-#endif
 		result= ip_write(icmp_port->icp_ipfd,
 			bf_bufsize(icmp_port->icp_write_pack));
 		if (result == NW_SUSPEND)
 		{
-#if DEBUG & 256
- { where(); printf("ip_write replied NW_SUSPEND\n"); }
-#endif
 			icmp_port->icp_flags |= ICPF_WRITE_SP;
 			return;
 		}
-#if DEBUG & 256
- { where(); printf("ip_write done\n"); }
-#endif
+		icmp_port->icp_flags &= ~ICPF_WRITE_IP;
 	}
-	icmp_port->icp_flags &= ~ICPF_WRITE_IP;
 }
 
-PRIVATE void icmp_buffree(priority, reqsize)
+PRIVATE void icmp_buffree(priority)
 int priority;
-size_t reqsize;
 {
 	acc_t *tmp_acc;
-	int donesomething,i;
+	int i;
 	icmp_port_t *icmp_port;
 
-	donesomething= 0;
-
-	if (priority < ICMP_PRI_QUEUE)
-		return;
-
-	while (bf_free_buffsize < reqsize)
+	if (priority == ICMP_PRI_QUEUE)
 	{
 		for (i=0, icmp_port= icmp_port_table; i<ICMP_PORT_NR;
 			i++, icmp_port++)
 		{
-			if (icmp_port->icp_head_queue)
+			while(icmp_port->icp_head_queue)
 			{
 				tmp_acc= icmp_port->icp_head_queue;
-				icmp_port->icp_head_queue= tmp_acc->
-					acc_ext_link;
+				icmp_port->icp_head_queue=
+					tmp_acc->acc_ext_link;
 				bf_afree(tmp_acc);
-				if (bf_free_buffsize >= reqsize)
-					break;
-				donesomething= 1;
 			}
 		}
-		if (!donesomething)
-			break;
 	}
 }
 
-static void icmp_dst_unreach(icmp_port, ip_pack, ip_hdr_len, ip_hdr, icmp_pack,
+#ifdef BUF_CONSISTENCY_CHECK
+PRIVATE void icmp_bufcheck()
+{
+	int i;
+	icmp_port_t *icmp_port;
+	acc_t *pack;
+
+	for (i= 0, icmp_port= icmp_port_table; i<ICMP_PORT_NR; 
+							i++, icmp_port++)
+	{
+		for (pack= icmp_port->icp_head_queue; pack; 
+			pack= pack->acc_ext_link)
+		{
+			bf_check_acc(pack);
+		}
+		bf_check_acc(icmp_port->icp_write_pack);
+	}
+}
+#endif
+
+PRIVATE void icmp_dst_unreach(icmp_port, ip_pack, ip_hdr_len, ip_hdr, icmp_pack,
 	icmp_len, icmp_hdr)
 icmp_port_t *icmp_port;
 acc_t *ip_pack;
@@ -652,12 +678,12 @@ icmp_hdr_t *icmp_hdr;
 {
 	acc_t *old_ip_pack;
 	ip_hdr_t *old_ip_hdr;
+	int ip_port_nr;
+	ipaddr_t dst, mask;
 
 	if (icmp_len < 8 + IP_MIN_HDR_SIZE)
 	{
-#if DEBUG
- { where(); printf("dest unrch with wrong size\n"); }
-#endif
+		DBLOCK(1, printf("dest unrch with wrong size\n"));
 		return;
 	}
 	old_ip_pack= bf_cut (icmp_pack, 8, icmp_len-8);
@@ -666,33 +692,44 @@ icmp_hdr_t *icmp_hdr;
 
 	if (old_ip_hdr->ih_src != ip_hdr->ih_dst)
 	{
-#if DEBUG
- { where(); printf("dest unrch based on wrong packet\n"); }
-#endif
+		DBLOCK(1, printf("dest unrch based on wrong packet\n"));
 		bf_afree(old_ip_pack);
 		return;
 	}
 
+	ip_port_nr= icmp_port->icp_ipport;
+
 	switch(icmp_hdr->ih_code)
 	{
 	case ICMP_NET_UNRCH:
-		ipr_destunrch (old_ip_hdr->ih_dst,
-			ip_get_netmask(old_ip_hdr->ih_dst), IPR_UNRCH_TIMEOUT);
-		break;
-	case ICMP_HOST_UNRCH:
-		ipr_destunrch (old_ip_hdr->ih_dst, (ipaddr_t)-1,
+		dst= old_ip_hdr->ih_dst;
+		mask= ip_get_netmask(dst);
+		ipr_destunrch (ip_port_nr, dst & mask, mask,
 			IPR_UNRCH_TIMEOUT);
 		break;
+	case ICMP_HOST_UNRCH:
+		ipr_destunrch (ip_port_nr, old_ip_hdr->ih_dst, (ipaddr_t)-1,
+			IPR_UNRCH_TIMEOUT);
+		break;
+	case ICMP_PORT_UNRCH:
+		/* At the moment we don't do anything with this information.
+		 * It should be handed to the appropriate transport layer.
+		 */
+		break;
 	default:
-#if DEBUG
- { where(); printf("got strange code: %d\n", icmp_hdr->ih_code); }
-#endif
+		DBLOCK(1, printf("icmp_dst_unreach: got strange code %d from ",
+			icmp_hdr->ih_code);
+			writeIpAddr(ip_hdr->ih_src);
+			printf("; original destination: ");
+			writeIpAddr(old_ip_hdr->ih_dst);
+			printf("; protocol: %d\n",
+			old_ip_hdr->ih_proto));
 		break;
 	}
 	bf_afree(old_ip_pack);
 }
 
-static void icmp_time_exceeded(icmp_port, ip_pack, ip_hdr_len, ip_hdr,
+PRIVATE void icmp_time_exceeded(icmp_port, ip_pack, ip_hdr_len, ip_hdr,
 	icmp_pack, icmp_len, icmp_hdr)
 icmp_port_t *icmp_port;
 acc_t *ip_pack;
@@ -704,12 +741,11 @@ icmp_hdr_t *icmp_hdr;
 {
 	acc_t *old_ip_pack;
 	ip_hdr_t *old_ip_hdr;
+	int ip_port_nr;
 
 	if (icmp_len < 8 + IP_MIN_HDR_SIZE)
 	{
-#if DEBUG
- { where(); printf("time exceeded with wrong size\n"); }
-#endif
+		DBLOCK(1, printf("time exceeded with wrong size\n"));
 		return;
 	}
 	old_ip_pack= bf_cut (icmp_pack, 8, icmp_len-8);
@@ -718,28 +754,31 @@ icmp_hdr_t *icmp_hdr;
 
 	if (old_ip_hdr->ih_src != ip_hdr->ih_dst)
 	{
-#if DEBUG
- { where(); printf("time exceeded based on wrong packet\n"); }
-#endif
+		DBLOCK(1, printf("time exceeded based on wrong packet\n"));
 		bf_afree(old_ip_pack);
 		return;
 	}
 
+	ip_port_nr= icmp_port->icp_ipport;
+
 	switch(icmp_hdr->ih_code)
 	{
 	case ICMP_TTL_EXC:
-		ipr_ttl_exc (old_ip_hdr->ih_dst, (ipaddr_t)-1,
+		ipr_ttl_exc (ip_port_nr, old_ip_hdr->ih_dst, (ipaddr_t)-1,
 			IPR_TTL_TIMEOUT);
 		break;
+	case ICMP_FRAG_REASSEM:
+		/* Ignore reassembly time-outs. */
+		break;
 	default:
-		where();
-		printf("got strange code: %d\n", icmp_hdr->ih_code);
+		DBLOCK(1, printf("got strange code: %d\n",
+			icmp_hdr->ih_code));
 		break;
 	}
 	bf_afree(old_ip_pack);
 }
 
-static void icmp_router_advertisement(icmp_port, icmp_pack, icmp_len, icmp_hdr)
+PRIVATE void icmp_router_advertisement(icmp_port, icmp_pack, icmp_len, icmp_hdr)
 icmp_port_t *icmp_port;
 acc_t *icmp_pack;
 int icmp_len;
@@ -753,65 +792,61 @@ icmp_hdr_t *icmp_hdr;
 
 	if (icmp_len < 8)
 	{
-#if DEBUG
- { where(); printf("router advertisement with wrong size (%d)\n", icmp_len); }
-#endif
+		DBLOCK(1,
+		printf("router advertisement with wrong size (%d)\n",
+			icmp_len));
 		return;
 	}
 	if (icmp_hdr->ih_code != 0)
 	{
-#if DEBUG
- { where(); printf("router advertisement with wrong code (%d)\n", 
-							icmp_hdr->ih_code); }
-#endif
+		DBLOCK(1,
+		printf("router advertisement with wrong code (%d)\n", 
+			icmp_hdr->ih_code));
 		return;
 	}
 	entries= icmp_hdr->ih_hun.ihh_ram.iram_na;
 	entry_size= icmp_hdr->ih_hun.ihh_ram.iram_aes * 4;
 	if (entries < 1)
 	{
-#if DEBUG
- { where(); printf("router advertisement with wrong number of entries (%d)\n", 
-							entries); }
-#endif
+		DBLOCK(1, printf(
+		"router advertisement with wrong number of entries (%d)\n", 
+			entries));
 		return;
 	}
 	if (entry_size < 8)
 	{
-#if DEBUG
- { where(); printf("router advertisement with wrong entry size (%d)\n", 
-							entry_size); }
-#endif
+		DBLOCK(1, printf(
+		"router advertisement with wrong entry size (%d)\n", 
+			entry_size));
 		return;
 	}
 	if (icmp_len < 8 + entries * entry_size)
 	{
-#if DEBUG
- { where(); printf("router advertisement with wrong size\n"); 
-	printf("\t(entries= %d, entry_size= %d, icmp_len= %d)\n", entries,
-						entry_size, icmp_len); }
-#endif
+		DBLOCK(1,
+			printf("router advertisement with wrong size\n"); 
+			printf(
+			"\t(entries= %d, entry_size= %d, icmp_len= %d)\n",
+			entries, entry_size, icmp_len));
 		return;
 	}
 	lifetime= ntohs(icmp_hdr->ih_hun.ihh_ram.iram_lt);
 	if (lifetime > 9000)
 	{
-#if DEBUG
- { where(); printf("router advertisement with wrong lifetime (%d)\n",
-								lifetime); }
-#endif
+		DBLOCK(1, printf(
+			"router advertisement with wrong lifetime (%d)\n",
+			lifetime));
 		return;
 	}
 	for (i= 0, bufp= (char *)&icmp_hdr->ih_dun.uhd_data[0]; i< entries; i++,
 		bufp += entry_size)
 	{
-		ipr_add_route(HTONL(0L), HTONL(0L), *(ipaddr_t *)bufp,
-			icmp_port->icp_ipport, lifetime * HZ, 1, 0, 
-			ntohl(*(i32_t *)(bufp+4)));
+		ipr_add_oroute(icmp_port->icp_ipport, HTONL(0L), HTONL(0L), 
+			*(ipaddr_t *)bufp, lifetime * HZ, 1, 0, 
+			ntohl(*(i32_t *)(bufp+4)), NULL);
 	}
 }
 		
-static void icmp_redirect(icmp_port, ip_hdr, icmp_pack, icmp_len, icmp_hdr)
+PRIVATE void icmp_redirect(icmp_port, ip_hdr, icmp_pack, icmp_len, icmp_hdr)
 icmp_port_t *icmp_port;
 ip_hdr_t *ip_hdr;
 acc_t *icmp_pack;
@@ -820,38 +855,133 @@ icmp_hdr_t *icmp_hdr;
 {
 	acc_t *old_ip_pack;
 	ip_hdr_t *old_ip_hdr;
-	int port;
+	int ip_port_nr;
+	ipaddr_t dst, mask;
 
 	if (icmp_len < 8 + IP_MIN_HDR_SIZE)
 	{
-#if DEBUG
- { where(); printf("redirect with wrong size\n"); }
-#endif
+		DBLOCK(1, printf("redirect with wrong size\n"));
 		return;
 	}
 	old_ip_pack= bf_cut (icmp_pack, 8, icmp_len-8);
 	old_ip_pack= bf_packIffLess(old_ip_pack, IP_MIN_HDR_SIZE);
 	old_ip_hdr= (ip_hdr_t *)ptr2acc_data(old_ip_pack);
 
-	port= icmp_port->icp_ipport;
+	ip_port_nr= icmp_port->icp_ipport;
 
 	switch(icmp_hdr->ih_code)
 	{
 	case ICMP_REDIRECT_NET:
-		ipr_redirect (old_ip_hdr->ih_dst,
-			ip_get_netmask(old_ip_hdr->ih_dst),
-			ip_hdr->ih_src, icmp_hdr->ih_hun.ihh_gateway, port, 
+		dst= old_ip_hdr->ih_dst;
+		mask= ip_get_netmask(dst);
+		ipr_redirect (ip_port_nr, dst & mask, mask,
+			ip_hdr->ih_src, icmp_hdr->ih_hun.ihh_gateway, 
 			IPR_REDIRECT_TIMEOUT);
 		break;
 	case ICMP_REDIRECT_HOST:
-		ipr_redirect (old_ip_hdr->ih_dst, (ipaddr_t)-1,
-			ip_hdr->ih_src, icmp_hdr->ih_hun.ihh_gateway, port, 
+		ipr_redirect (ip_port_nr, old_ip_hdr->ih_dst, (ipaddr_t)-1,
+			ip_hdr->ih_src, icmp_hdr->ih_hun.ihh_gateway, 
 			IPR_REDIRECT_TIMEOUT);
 		break;
 	default:
-		where();
-		printf("got strange code: %d\n", icmp_hdr->ih_code);
+		DBLOCK(1, printf("got strange code: %d\n",
+			icmp_hdr->ih_code));
 		break;
 	}
 	bf_afree(old_ip_pack);
 }
+
+PRIVATE acc_t *icmp_err_pack(pack, icmp_hdr)
+acc_t *pack;
+icmp_hdr_t **icmp_hdr;
+{
+	ip_hdr_t *ip_hdr;
+	acc_t *ip_pack, *icmp_pack, *tmp_pack;
+	int ip_hdr_len, icmp_hdr_len;
+	size_t size;
+	ipaddr_t dest, netmask;
+	nettype_t nettype;
+
+	pack= bf_packIffLess(pack, IP_MIN_HDR_SIZE);
+	ip_hdr= (ip_hdr_t *)ptr2acc_data(pack);
+
+	/* If the IP protocol is ICMP or the fragment offset is non-zero,
+	 * drop the packet. Also check if the source address is valid.
+	 */
+	if (ip_hdr->ih_proto == IPPROTO_ICMP || 
+		(ntohs(ip_hdr->ih_flags_fragoff) & IH_FRAGOFF_MASK) != 0)
+	{
+		bf_afree(pack);
+		return NULL;
+	}
+	dest= ip_hdr->ih_src;
+	nettype= ip_nettype(dest);
+	netmask= ip_netmask(nettype);
+	if ((nettype != IPNT_CLASS_A && nettype != IPNT_LOCAL &&
+		nettype != IPNT_CLASS_B && nettype != IPNT_CLASS_C) ||
+		(dest & ~netmask) == 0 || (dest & ~netmask) == ~netmask)
+	{
+#if !CRAMPED
+		printf("icmp_err_pack: invalid source address: ");
+		writeIpAddr(dest);
+		printf("\n");
+#endif
+		bf_afree(pack);
+		return NULL;
+	}
+
+	/* Take the IP header and the first 64 bits of user data. */
+	size= ntohs(ip_hdr->ih_length);
+	ip_hdr_len= (ip_hdr->ih_vers_ihl & IH_IHL_MASK) << 2;
+	if (size < ip_hdr_len || bf_bufsize(pack) < size)
+	{
+#if !CRAMPED
+		printf("icmp_err_pack: wrong packet size:\n");
+		printf("\thdrlen= %d, ih_length= %d, bufsize= %d\n",
+			ip_hdr_len, size, bf_bufsize(pack));
+#endif
+		bf_afree(pack);
+		return NULL;
+	}
+	if (ip_hdr_len + 8 < size)
+		size= ip_hdr_len+8;
+	tmp_pack= bf_cut(pack, 0, size);
+	bf_afree(pack);
+	pack= tmp_pack;
+	tmp_pack= NULL;
+
+	/* Create a minimal size ICMP hdr. */
+	icmp_hdr_len= offsetof(icmp_hdr_t, ih_dun);
+	icmp_pack= bf_memreq(icmp_hdr_len);
+	pack= bf_append(icmp_pack, pack);
+	size += icmp_hdr_len;
+	pack= bf_packIffLess(pack, icmp_hdr_len);
+	*icmp_hdr= (icmp_hdr_t *)ptr2acc_data(pack);
+	(*icmp_hdr)->ih_type= 0;
+	(*icmp_hdr)->ih_code= 0;
+	(*icmp_hdr)->ih_chksum= 0;
+	(*icmp_hdr)->ih_hun.ihh_unused= 0;
+	(*icmp_hdr)->ih_chksum= ~icmp_pack_oneCsum(pack);
+
+	/* Create an IP header */
+	ip_hdr_len= IP_MIN_HDR_SIZE;
+
+	ip_pack= bf_memreq(ip_hdr_len);
+	ip_hdr= (ip_hdr_t *)ptr2acc_data(ip_pack);
+
+	ip_hdr->ih_vers_ihl= ip_hdr_len >> 2;
+	ip_hdr->ih_tos= 0;
+	ip_hdr->ih_length= htons(ip_hdr_len + size);
+	ip_hdr->ih_flags_fragoff= 0;
+	ip_hdr->ih_ttl= ICMP_DEF_TTL;
+	ip_hdr->ih_proto= IPPROTO_ICMP;
+	ip_hdr->ih_dst= dest;
+
+	assert(ip_pack->acc_next == NULL);
+	ip_pack->acc_next= pack;
+	return ip_pack;
+}
+
+/*
+ * $PchId: icmp.c,v 1.8 1996/12/17 07:53:34 philip Exp $
+ */
